;;; Roman/theme.jl
;;; Created by Herman Roozenbeek <hermanr@mailroom.com>

(defgroup Roman-frame "Roman theme options"
  :group appearance)

(defcustom Roman:full-title 'nil
  "Use full width of window for title and buttons"
  :group (appearance Roman-frame)
  :type boolean)

;; (defvar roman-sticky-button-keymap (make-keymap))

(def-frame-class roman-sticky-button '((cursor . right_ptr))
  (bind-keys roman-sticky-button-keymap 
    "Button1-Click1" 'popup-window-menu
    "Button3-Off" 'toggle-window-sticky))

(let*
  (
;;    (font default-font)
    (font-colors (list (get-color "#847D7B")
                       "black"))

    (is-shaded
      (lambda (w)
        (window-get w 'shaded)))

    (is-not-shaded
      (lambda (w)
        (if (is-shaded w) nil t)))

    (is-transient
      (lambda (w)
        (let
          ((wtype (window-type w)))
          (if (or (eq wtype 'transient) 
                  (eq wtype 'shaped-transient))
            t
            nil))))

    (is-not-transient
      (lambda (w)
        (if (is-transient w) nil t)))

    (max-width
      (lambda (w)
        (if (is-transient w) 58 106)))

    (title-width
      (lambda (w)
        (let
          ((w-width (+ 10 (car (window-dimensions w)))))
        (if (eq Roman:full-title nil)
               (max 0 (min (- w-width (max-width w)) (+ 24 (text-width (window-name w) default-font))))
               (max 0 (- w-width (max-width w)))))))

    (roman-bevel
      (lambda (im b)
        (let
          ((image (make-image im)))
          (bevel-image image 2 b)
          image)))

    (roman-bevel-up
      (lambda (im)
        (roman-bevel im 't)))

    (roman-bevel-down
      (lambda (im)
        (roman-bevel im 'nil)))

    (redraw-this-frame
      (lambda (w)
        (if (eq (window-get w 'current-frame-style) 'Roman) 
          (rebuild-frame w))))

    (rebuild
      (lambda()
        (rebuild-frames-with-style 'Roman)))

    (left-title-image (make-image "title-left.png"))
    (full-right-title-image (make-image "full-title-right.png"))
    (short-right-title-image (make-image "short-title-right.png"))
    (right-title-image (lambda (w)
                          (if (and (eq Roman:full-title nil)
                                   (is-not-shaded w))
                            (if (eq (title-width w) (- (car (window-dimensions w)) (- (max-width w) 10)))
                               full-right-title-image
                               short-right-title-image)
                            full-right-title-image)))
    (title-image (make-image "mid-title.png"))
    (sticky-images (list (make-image "sticky-inactive.png")
                         (make-image "sticky-active.png")))
    (default-icon (make-image "default_icon.png"))
    (icon-image (lambda (w)
                        (if (window-get w 'sticky)
                            sticky-images
                            (let ((icon (window-icon-image w)))
                                 (if (eq icon nil) default-icon (scale-image icon 16 16)))
                            )
                        ))
    (menu-images (list (make-image "menu-inactive.png")
                         (make-image "menu-active.png")
                         (roman-bevel-up "menu-active.png")
                         (roman-bevel-down "menu-active.png")))
    (button-images (list (make-image "button-background.png")
                         (make-image "button-background.png")
                         (roman-bevel-up "button-background.png")
                         (roman-bevel-down "button-background.png")))
    (close-images (list (make-image "close-inactive.png") 
                        (make-image "close-active.png")))
    (maximize-images (list (make-image "maximize-inactive.png")
                           (make-image "maximize-active.png")))
    (iconify-images (list (make-image "iconify-inactive.png")
                          (make-image "iconify-active.png")))
    (bottom-image (make-image "bottom-bar.png"))
    (top-image (make-image "top-bar.png"))
    (top-left-image (make-image "top-left.png"))
    (top-right-image (make-image "top-right.png"))
    (bottom-left-image (make-image "bottom-left.png"))
    (bottom-right-image (make-image "bottom-right.png"))
    (left-image (make-image "left-bar.png"))
    (right-image (make-image "right-bar.png"))

    (normal-frame `(
        ((background . ,top-image)
         (top-edge . -5)
         (left-edge . ,(lambda (w) (+ (title-width w) (- (max-width w) 10))))
         (right-edge . 0)
         (hidden . ,is-shaded)
         (class . title))
        ((background . ,left-image)
         (left-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (hidden . ,is-shaded)
         (class . left-border))
        ((background . ,top-right-image)
         (right-edge . -5)
         (top-edge . -5)
         (hidden . ,is-shaded)
         (class . title))
        ((background . ,right-image)
         (right-edge . -5)
         (top-edge . 0)
         (bottom-edge . 0)
         (hidden . ,is-shaded)
         (class . right-border))
        ((background . ,bottom-image)
         (left-edge . 0)
         (right-edge . 0)
         (bottom-edge . -5)
         (hidden . ,is-shaded)
         (class . bottom-border))
        ((background . ,bottom-image)
         (left-edge . 0)
         (width . ,(lambda (w) (+ (title-width w) (- (max-width w) 10))))
         (top-edge . 0)
         (hidden . ,is-not-shaded)
         (class . title))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (bottom-edge . -5)
         (hidden . ,is-shaded)
         (class . bottom-left-corner))
        ((background . ,bottom-left-image)
         (left-edge . -5)
         (top-edge . 0)
         (hidden . ,is-not-shaded)
         (class . title))
        ((background . ,bottom-right-image)
         (right-edge . -5)
         (bottom-edge . -5)
         (hidden . ,is-shaded)
         (class . bottom-right-corner))
        ((background . ,bottom-right-image)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) (- (max-width w) 5) (title-width w))))
         (top-edge . 0)
         (hidden . ,is-not-shaded)
         (class . title))
        ((background . ,left-title-image)
         (left-edge . -5)
         (top-edge . -24)
         (class . title))
        ((background . ,right-title-image)
         (right-edge . ,(lambda (w) (- (car (window-dimensions w)) (- (max-width w) 5) (title-width w))))
         (top-edge . -24)
         (class . title))
        ((background . ,title-image)
         (left-edge . 0)
         (width . ,(lambda (w) (+ (title-width w) (- (max-width w) 10))))
         (top-edge . -24)
         (class . title))
        ((background . ,title-image)
         (foreground . ,font-colors)
         (text . ,window-name)
         (top-edge . -24)
         (left-edge . 24)
         (width . ,title-width)
         (height . 24)
         (x-justify . center)
         (y-justify . center)
         (class . title))
        ((background . ,button-images)
         (foreground . ,icon-image)
         (x-justify . center)
         (y-justify . center)
         (left-edge . 0)
         (top-edge . -22)
         (removable . t)
         (class . roman-sticky-button))
        ((background . ,button-images)
         (foreground . ,iconify-images)
         (x-justify . center)
         (y-justify . center)
         (left-edge . ,(lambda (w) (+ (title-width w) 24)))
         (top-edge . -22)
         (removable . t)
         (hidden . ,is-transient)
         (class . iconify-button))
        ((background . ,button-images)
         (foreground . ,maximize-images)
         (x-justify . center)
         (y-justify . center)
         (left-edge . ,(lambda (w) (+ (title-width w) 48)))
         (top-edge . -22)
         (removable . t)
         (hidden . ,is-transient)
         (class . maximize-button))
        ((background . ,button-images)
         (foreground . ,close-images)
         (x-justify . center)
         (y-justify . center)
         (left-edge . ,(lambda (w) (+ (title-width w) (if (is-transient w) 24 72))))
         (top-edge . -22)
         (removable . t)
         (class . close-button))
      )))

  (add-frame-style 'Roman
    (lambda (w type)
      (case type 
            ((default) normal-frame)
            ((shaded) normal-frame)
            ((transient) normal-frame)
            ((shaded-transient) normal-frame))))
            
;  (mapc (lambda (sym)
;         (custom-set-property sym ':after-set rebuild))
;        '(Roman:full-title))

  (custom-set-property 'Roman:full-title ':after-set rebuild)

  (call-after-property-changed 'WM_NAME redraw-this-frame)
  (call-after-property-changed 'WM_HINTS redraw-this-frame)
  (call-after-state-changed '(sticky) redraw-this-frame)
)
